/* Xscanner: A generic scanner frontend for Unix written using Xforms.
 * Kees Lemmens; Nov. 1997
 * Modified Jan 2001 to support SANE as well
 */

// #define DEBUG 1

/* Add your own backend specifications if necessary !! */
 #define SANE_HPSCAN  1
// #define SANE_EPSON   1
// #define HPSCANPBM 1

#include "xscanner.h"
#include "version.h"

#define XSCANRESOURCES  "xscanrc"

#define SCANPRINT "pnmtops -rle %s | lpps"
#define SCANVIEW  "xv %s"
/* #define PPMTOGIF  "ppmquant 256 %s | ppmtogif > %s" */
#define PPMTOJPG  "cjpeg -quality 80 %s > %s" /* assumes JPEG tools */
#define PPMTOPNG  "convert %s %s"             /* assumes ImageMagick */
#define PPMTOPS   "pnmtops -rle %s > %s"
#define SPOOLDIR  "/tmp"

/* SANE HP Scanjet II (only grayscale) backend options */
#ifdef SANE_HPSCAN
# define DEFAULT "Sane with Hp backend"

# define SCANCMD "scanimage" /* SANE cmdline tool for LINUX */
# define HASBRIGHTNESS
# define BRGTNS_OFFSET "0"
# define BRGTNS_FACTOR "127"

# define HASCONTRAST 
# define CNTRST_OFFSET "0"
# define CNTRST_FACTOR "127"

# define FORMATS " Color | Grayscale | Halftone | Lineart "
# define PREVIEWCMD "%s --mode %s --resolution 75 -y %f "            \
   "--preview=yes --format=pnm --contrast %.0f --brightness %.0f | " \
     "pnmscale 0.46 | ppmquant 256 | ppmtoxpm > %s"
# define FINALCMD  "%s --mode %s --resolution %d -x %f -y %f "       \
   "--contrast %.0f --brightness %.0f > %s"
#endif

/* SANE Epson Perfection 610 scanner backend options */
#ifdef SANE_EPSON
# define DEFAULT "Sane with Epson backend"

# define SCANCMD "scanimage" /* SANE cmdline tool for LINUX */
# define HASBRIGHTNESS   // External !
# define BRGTNS_OFFSET "0"
# define BRGTNS_FACTOR "100"

# define HASCONTRAST  // External !
# define CNTRST_OFFSET "0"
# define CNTRST_FACTOR "100"

/*
 Note that I use a simple external program for brightness and contrast
 control as my scanner (Perfection 610) does not support it : */

# define FORMATS " Color | Gray | Binary "
# define PREVIEWCMD "%s --mode %s --resolution 75 -y %f "   \
   "--preview=yes --format=pnm | pnmscale 0.46 | "          \
     "epsonbright %.0f %.0f | ppmquant 256 | ppmtoxpm > %s"
# define FINALCMD   "%s --mode %s --resolution %d -x %f -y %f | " \
   "epsonbright %.0f %.0f > %s"
#endif

/* Add your own SANE backend defs here if you like : */

/* Support for good old HPSCANPBM backend */
#ifdef HPSCANPBM
# define DEFAULT "Good old 'hpscanpbm' backend"

# define SCANCMD "hpscanpbm" /* HP Scanjet cmdline tool for HPUX and LINUX */
# define HASBRIGHTNESS
# define BRGTNS_OFFSET "0"
# define BRGTNS_FACTOR "1"

# define HASCONTRAST 
# define CNTRST_OFFSET "0"
# define CNTRST_FACTOR "1"

# define FORMATS " greyscale | color | thresholded | dithered "
# define PREVIEWCMD "%s -format %s -dpi 75 -mm "              \
   "-xres 300 -yres 400 -height %f -cont %f -bright %f | "    \
     "ppmquant 256 | ppmtoxpm > %s"
# define FINALCMD   "%s -format %s -dpi %d -mm "              \
   "-width %f -height %f -cont %f -bright %f -o %s"
#endif

/* Relation between preview area and real scan area :
 It's slightly more than A4 dims to ensure we really 
 catch everything we selected with the crosshair.
 */
#define XFACTOR  216
#define YFACTOR  298

#define MAXCMD      4000  // maximum length for an arbitrary command
#define MAXSTRING    256  // maximum string length
#define MAXLINE       80  // max length of a line in the logviewer

typedef struct 
{
   char SpoolDir [MAXSTRING];
   char ScanPrint[MAXSTRING];
   char ScanView [MAXSTRING];
   char PpmToJpg [MAXSTRING];
   char PpmToPng [MAXSTRING];
   char PpmToPs  [MAXSTRING];
   char Formats  [MAXSTRING];
   char ScanCmd  [MAXSTRING];
   char PreviewCmd[MAXSTRING];
   char FinalCmd  [MAXSTRING];
   char BrightnessOffset[MAXSTRING];
   char BrightnessFactor[MAXSTRING];
   char ContrastOffset  [MAXSTRING];
   char ContrastFactor  [MAXSTRING];

   char Version[MAXSTRING];
   
   char *Scanname;
   int  ExitFlag;
   int  SkipRCFlag;
} Xscanrc;

extern FD_Xscanner *fd_Xscanner;
extern FD_LogView *fd_LogView;
extern FD_SystemWin *fd_SystemWin;
extern Xscanrc *info;

/* External function declarations */

char    *BuildResourceFile();
char    *ReadString(const char *label,char *string,FILE *rcfile);
void    LoadUserInfo(Xscanrc *info);
void    SaveUserInfo(const Xscanrc *info);
void    InitSystemWin(FD_SystemWin *fd_SystemWin);
void    RedirectIO(FILE *iohandle, int type);
void    SpawnCmd(char *cmdbuffer,int flag);
void    RemoveTmpdir(char *dirname);
void    LogViewOpen(FL_OBJECT *ob, long data);
